@@ -1,8 +1,9 @@
 /* Defaults.m
  *  
- * Copyright (C) 2006 Free Software Foundation, Inc.
+ * Copyright (C) 2006-2013 Free Software Foundation, Inc.
  *
  * Author: Enrico Sersale <enrico@imago.ro>
+ *         Riccardo Mottola <rm@gnu.org>
  * Date: February 2006
  *
  * This file is part of the GNUstep "Defaults" Preference Pane
@@ -22,10 +23,10 @@
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02111 USA.
  */
 
-#include <AppKit/AppKit.h>
-#include "Defaults.h"
-// For INT_MAX and INT_MIN
-#include <limits.h>
+#import <AppKit/AppKit.h>
+#import "Defaults.h"
+
+#include <limits.h>  // For INT_MAX and INT_MIN
 
 @implementation Defaults
 
@@ -36,21 +37,22 @@
   TEST_RELEASE (boolEditorBox);
   TEST_RELEASE (numberEditorBox);
   TEST_RELEASE (arrayEditorBox);
+  TEST_RELEASE (listEditorBox);
   
-	[super dealloc];
+  [super dealloc];
 }
 
 - (void)mainViewDidLoad
 {
   if (loaded == NO) {
     NSUserDefaults *defaults = [NSUserDefaults standardUserDefaults];
-    NSBundle *bundle = [NSBundle bundleForClass: [self class]];
+    NSBundle *bundle = [self bundle];
     NSString *dictpath = [bundle pathForResource: @"Defaults" ofType: @"plist"];
     NSDictionary *dict = [NSDictionary dictionaryWithContentsOfFile: dictpath];
     NSArray *keys = [[dict allKeys] sortedArrayUsingSelector: @selector(compare:)];
     id cell;
     float fonth;
-    unsigned i;
+    NSUInteger i;
     
     RETAIN (stringEditorBox);
     [stringEditorBox removeFromSuperview];
@@ -60,7 +62,8 @@
     [numberEditorBox removeFromSuperview];
     RETAIN (arrayEditorBox);
     [arrayEditorBox removeFromSuperview];
-    
+    RETAIN (listEditorBox);
+    [listEditorBox removeFromSuperview];
     RELEASE (editorsWin);
     
     [defaults synchronize];
@@ -70,15 +73,18 @@
       NSString *defname = [keys objectAtIndex: i];
       NSDictionary *info = [dict objectForKey: defname];
       NSString *category = [info objectForKey: @"category"];
-      NSString *description = [info objectForKey: @"description"];
+      NSString *description;
+      NSArray *values = [info objectForKey: @"values"];
       id defvalue = [info objectForKey: @"defaultvalue"];
       int edtype = [[info objectForKey: @"editor"] intValue];
       DefaultEntry *entry;
       
+      description = [bundle localizedStringForKey:defname value:@"Description not found" table:nil];
       entry = [[DefaultEntry alloc] initWithUserDefaults: defaults
                                                 withName: defname 
                                               inCategory: category
                                              description: description
+						  values: values
                                             defaultValue: defvalue 
                                              editorType: edtype];
       [defaultsEntries addObject: entry];
@@ -93,24 +99,24 @@
     fonth = [[cell font] defaultLineHeightForFont];
 
     namesMatrix = [[NSMatrix alloc] initWithFrame: NSMakeRect(0, 0, 100, 100)
-				            	              mode: NSRadioModeMatrix 
-                               prototype: cell
-			       							  numberOfRows: 0 
-                         numberOfColumns: 0];
+                                             mode: NSRadioModeMatrix 
+                                        prototype: cell
+                                     numberOfRows: 0 
+                                  numberOfColumns: 0];
     RELEASE (cell);                     
     [namesMatrix setIntercellSpacing: NSZeroSize];
     [namesMatrix setCellSize: NSMakeSize([namesScroll contentSize].width, fonth)];
     [namesMatrix setAutoscroll: YES];
-	  [namesMatrix setAllowsEmptySelection: YES];
+    [namesMatrix setAllowsEmptySelection: YES];
     [namesMatrix setTarget: self]; 
     [namesMatrix setAction: @selector(namesMatrixAction:)]; 
-	  [namesScroll setDocumentView: namesMatrix];	
+    [namesScroll setDocumentView: namesMatrix];	
     RELEASE (namesMatrix);
     
     for (i = 0; i < [defaultsEntries count]; i++) {
       DefaultEntry *entry = [defaultsEntries objectAtIndex: i];
       NSString *name = [entry name];
-      int count = [[namesMatrix cells] count];
+      NSUInteger count = [[namesMatrix cells] count];
       
       [namesMatrix insertRow: count];
       cell = [namesMatrix cellAtRow: count column: 0];   
@@ -122,6 +128,8 @@
         
     [descriptionView setFont: [NSFont systemFontOfSize: 10]];
     [descriptionView setDrawsBackground: NO];
+    // FIXME can't turn off this attribute in Gorm at present
+    [descriptionView setEditable: NO];
 
     currentEntry = nil;    
     loaded = YES;
@@ -132,7 +140,7 @@
     
     // Bool
     [boolEdPopup selectItemAtIndex: 0];
-    
+
     // Number
     [numberEdField setStringValue: @""];
     [numberEdField setDelegate: self];
@@ -146,34 +154,42 @@
     [arrayEdScroll setHasVerticalScroller: YES]; 
     
     arrayEdMatrix = [[NSMatrix alloc] initWithFrame: NSMakeRect(0, 0, 100, 100)
-				            	              mode: NSRadioModeMatrix 
-                               prototype: cell
-			       							  numberOfRows: 0 
-                         numberOfColumns: 0];
+                                               mode: NSRadioModeMatrix 
+                                          prototype: cell
+                                       numberOfRows: 0 
+                                    numberOfColumns: 0];
     [arrayEdMatrix setIntercellSpacing: NSZeroSize];
     [arrayEdMatrix setCellSize: NSMakeSize([arrayEdScroll contentSize].width, fonth)];
     [arrayEdMatrix setAutoscroll: YES];
-	  [arrayEdMatrix setAllowsEmptySelection: YES];
+    [arrayEdMatrix setAllowsEmptySelection: YES];
     [arrayEdMatrix setTarget: self]; 
     [arrayEdMatrix setAction: @selector(arrayEdMatrixAction:)]; 
-	  [arrayEdScroll setDocumentView: arrayEdMatrix];	
+    [arrayEdScroll setDocumentView: arrayEdMatrix];
     RELEASE (arrayEdMatrix);
+
+    // List
+    [listEdPopup selectItemAtIndex: 0];
+
   
     [self disableControls];
+
+    /* set some locales */
+    [categoryLabel setStringValue: [bundle localizedStringForKey:@"category" value:@"n/f" table:nil]];
+    [descriptionLabel setStringValue: [bundle localizedStringForKey:@"description" value:@"n/f" table:nil]];
   }
 }
 
 - (DefaultEntry *)entryWithName:(NSString *)name
 {
-  unsigned i;
+  NSUInteger i;
 
-  for (i = 0; i < [defaultsEntries count]; i++) {
-    DefaultEntry *entry = [defaultsEntries objectAtIndex: i];
-
-    if ([[entry name] isEqual: name]) {
-      return entry;
+  for (i = 0; i < [defaultsEntries count]; i++)
+    {
+      DefaultEntry *entry = [defaultsEntries objectAtIndex: i];
+      
+      if ([[entry name] isEqual: name])
+        return entry;
     }
-  }
     
   return nil;
 }
@@ -188,12 +204,14 @@
     int edtype;
     id defvalue;
     id usrvalue;
+    NSArray *values;
     id value;
        
     currentEntry = [self entryWithName: [cell stringValue]];
     edtype = [currentEntry editorType];
     defvalue = [currentEntry defaultValue];  
     usrvalue = [currentEntry userValue];
+    values = [currentEntry values];
     value = (usrvalue == nil) ? defvalue : usrvalue;
           
     [descriptionView setString: [currentEntry description]];
@@ -220,7 +238,7 @@
 
       case ARRAY_EDITOR:
         {
-          unsigned i;
+          NSUInteger i;
           
           [editorBox setContentView: arrayEditorBox];
           
@@ -230,7 +248,7 @@
           
           for (i = 0; i < [value count]; i++) {
             NSString *str = [value objectAtIndex: i];
-            int count = [[arrayEdMatrix cells] count];
+            NSUInteger count = [[arrayEdMatrix cells] count];
 
             [arrayEdMatrix insertRow: count];
             cell = [arrayEdMatrix cellAtRow: count column: 0];   
@@ -242,6 +260,17 @@
           [arrayEdDefaultRevert setEnabled: (usrvalue && ([usrvalue isEqual: defvalue] == NO))];
           break;    
         }
+
+    case LIST_EDITOR:
+      [editorBox setContentView: listEditorBox];
+      if (values && [values count] > 0)
+        {
+          [listEdPopup removeAllItems];
+          [listEdPopup addItemsWithTitles:values];
+          [listEdPopup selectItemWithTitle: value];
+        }
+      [listEdDefaultRevert setEnabled: (usrvalue && ([usrvalue isEqual: defvalue] == NO))];
+      break;
         
       default:
         break;
@@ -283,7 +312,7 @@
 
   if (usrvalue != nil) {
     if ([usrvalue isEqual: defvalue] == NO) {
-   //   NSLog(@"setting: %@ for: %@", [usrvalue description], defname);
+   //      NSLog(@"setting: %@ for: %@", [usrvalue description], defname);
       [domain setObject: usrvalue forKey: defname]; 
     } else {
    //   NSLog(@"removing: %@", defname);
@@ -483,7 +512,7 @@
     BOOL exists = NO;
     
     if (count > 0) {
-      unsigned i;
+      NSUInteger i;
       
       for (i = 0; i < count; i++) {
         NSString *cellstr = [[cells objectAtIndex: i] stringValue];
@@ -528,8 +557,8 @@
     id usrvalue = [currentEntry userValue];
     NSMutableArray *newvalue = [NSMutableArray array];  
     NSArray *cells;
-    int row, col;
-    unsigned i;
+    NSInteger row, col;
+    NSUInteger i;
     
     [arrayEdMatrix getRow: &row column: &col ofCell: cell];
     [arrayEdMatrix removeRow: row];
@@ -560,7 +589,7 @@
   id defvalue = [currentEntry defaultValue];
   id usrvalue = [currentEntry userValue];
   id cell;
-  unsigned i;
+  NSUInteger i;
        
   if ([arrayEdMatrix numberOfColumns] > 0) {        
     [arrayEdMatrix removeColumn: 0];
@@ -568,7 +597,7 @@
   
   for (i = 0; i < [defvalue count]; i++) {
     NSString *str = [defvalue objectAtIndex: i];
-    int count = [[arrayEdMatrix cells] count];
+    NSUInteger count = [[arrayEdMatrix cells] count];
 
     [arrayEdMatrix insertRow: count];
     cell = [arrayEdMatrix cellAtRow: count column: 0];   
@@ -593,7 +622,7 @@
   id usrvalue = [currentEntry userValue];
   NSArray *cells = [arrayEdMatrix cells];
   NSMutableArray *newvalue = [NSMutableArray array];
-  unsigned i;
+  NSUInteger i;
 
   for (i = 0; i < [cells count]; i++) {
     [newvalue addObject: [[cells objectAtIndex: i] stringValue]];
@@ -609,6 +638,59 @@
   RELEASE (arp);  
 }
 
+//
+// List
+//
+- (IBAction)listPopupAction:(id)sender
+{
+  NSString *str = [listEdPopup titleOfSelectedItem];
+  id defvalue = [currentEntry defaultValue];
+  id usrvalue = [currentEntry userValue];
+
+  [listEdDefaultRevert setEnabled: ([str isEqual: defvalue] == NO)];  
+
+  if (usrvalue)
+    {
+      [listEdSet setEnabled: ([str isEqual: usrvalue] == NO)];
+    }
+  else
+    {
+      [listEdSet setEnabled: ([str isEqual: defvalue] == NO)];
+    }
+}
+
+- (IBAction)listDefaultRevertAction:(id)sender
+{
+  id defvalue = [currentEntry defaultValue];
+  id usrvalue = [currentEntry userValue];
+
+  [listEdPopup selectItemWithTitle: defvalue];
+  [listEdDefaultRevert setEnabled: NO];
+
+  if (usrvalue)
+    {
+      [listEdSet setEnabled: ([usrvalue isEqual: defvalue] == NO)];  
+    }
+  else
+    {
+      [listEdSet setEnabled: NO];  
+    }
+}
+
+- (IBAction)listSetAction:(id)sender
+{
+  NSString *str = [listEdPopup titleOfSelectedItem];  
+  id usrvalue = [currentEntry userValue];
+
+  if ((usrvalue == nil) || ([str isEqual: usrvalue] == NO))
+    {
+      [currentEntry setUserValue: str];
+      [self updateDefaults];
+    }
+  
+  [listEdSet setEnabled: NO];
+}
+
 @end
 
 
@@ -621,14 +703,15 @@
   RELEASE (description);
   TEST_RELEASE (userValue);
   RELEASE (defaultValue);
-  
-	[super dealloc];
+  RELEASE (values);
+  [super dealloc];
 }
 
 - (id)initWithUserDefaults:(NSUserDefaults *)defaults
                   withName:(NSString *)dfname
                 inCategory:(NSString *)cat
                description:(NSString *)desc
+		    values:(NSArray *)vals
               defaultValue:(id)dval
                 editorType:(int)edtype
 {
@@ -639,6 +722,7 @@
     ASSIGN (category, cat);
     ASSIGN (description, desc);
     ASSIGN (defaultValue, dval);
+    ASSIGN (values, vals);
     editorType = edtype;
     
     userValue = [defaults objectForKey: name];
@@ -682,6 +766,11 @@
   }
 }
 
+- (NSArray *)values
+{
+  return values;
+}
+
 - (int)editorType
 {
   return editorType;
@@ -689,6 +778,3 @@
 
 @end
 
-
-
-
